mgMappingLocalizer = {
  dict : null,
  reverse : null,
  bundle : null,

  init : function() {
    // initialize localization dictionary
    var srv = Components.classes["@mozilla.org/intl/stringbundle;1"].getService()
              .QueryInterface(Components.interfaces.nsIStringBundleService);
    this.bundle = srv.createBundle("chrome://mozgest/locale/mozgest.properties");

    this.dict = new Array();
    this.dict["R"] = this.bundle.GetStringFromName("abbreviation.right");
    this.dict["L"] = this.bundle.GetStringFromName("abbreviation.left");
    this.dict["U"] = this.bundle.GetStringFromName("abbreviation.up");
    this.dict["D"] = this.bundle.GetStringFromName("abbreviation.down");
    this.dict["1"] = this.bundle.GetStringFromName("abbreviation.d1");
    this.dict["3"] = this.bundle.GetStringFromName("abbreviation.d3");
    this.dict["7"] = this.bundle.GetStringFromName("abbreviation.d7");
    this.dict["9"] = this.bundle.GetStringFromName("abbreviation.d9");

    this.reverse = new Array();
    for (var loc in this.dict) {
      this.reverse[this.dict[loc]] = loc;
    }
  },

  localize : function(orig) {
    if(!this.dict) this.init();
    return this.convert(orig, this.dict);
  },

  deLocalize : function(localized) {
    if(!this.dict) this.init();
    return this.convert(localized, this.reverse);
  },

  convert : function(str, table) {
    if (!str) return "";
    var trans = "";
    for (var i = 0; i < str.length; i++) {
      var o = str.substring(i, i+1);
      if(table[o]) {
        trans = trans + table[o];
      } else {
        trans = trans + o;
      }
    }
    return trans;
  },

  parseRDF : function(property, target) {
    if (!target) return target;
    if (property.Value == mgStorage.NS + "code") {
      var orig = target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
      locNode = mgStorage.RDF.GetLiteral(this.localize(orig));
      return locNode;
    } else {
      return target;
    }
  },

  getString : function(aName) {
    if(!this.bundle) this.init();
    return this.bundle.GetStringFromName(aName);
  },
}

mgLocalizingDS = function() {
  this.init();
}

mgLocalizingDS.prototype = {
  URI : "urn:mozgest:localizing-user-store",
  prefix : "urn:mozgest-localized:",
  ds : null,
  source : null,
  loc : null,

  init : function() {
    this.ds = Components.classes['@mozilla.org/rdf/datasource;1?name=in-memory-datasource']
              .createInstance(Components.interfaces.nsIRDFDataSource);

    // store a reference to our localizer (needed when window is closing)
    this.loc = mgMappingLocalizer;

    this.source = mgStorage.getUserRDF();
    this.source.AddObserver(this);

    this.copyUserStore();
  },
  
  copyUserStore : function() {
    var elements = this.source.GetAllResources();
    while (elements.hasMoreElements()) {
      var el = elements.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
      // only process Seqs
      if(mgStorage.RDFCUtils.IsSeq(this.source, el)) {
        var seqRes = el;
        var sourceSeq = mgStorage.newContainer();
        sourceSeq.Init(this.source, el);
        var targetSeq = mgStorage.newContainer();
        try {
          targetSeq.Init(this.ds, seqRes);
        } catch (e) {
          mgStorage.RDFCUtils.MakeSeq(this.ds, seqRes);
          targetSeq.Init(this.ds, seqRes);
        }
        var mappingsEnum = sourceSeq.GetElements();
        while (mappingsEnum.hasMoreElements()) {
          var mapping = mappingsEnum.getNext()
                        .QueryInterface(Components.interfaces.nsIRDFResource);
          this.copyMapping(mapping, targetSeq);
        }
      }
    }
  },

  copyMapping : function(aMapping, aTargetSeq) {
    var mUri = aMapping.Value;

    var oCode = mgStorage.getProperty(this.source, aMapping, "code");
    var oFunc = mgStorage.getProperty(this.source, aMapping, "function");
    var oName = mgStorage.getProperty(this.source, aMapping, "name")
                || this.loc.getString(oFunc);

    var node = mgStorage.RDF.GetResource(this.prefix + mUri);
    aTargetSeq.AppendElement(node, true);
    mgStorage.assertLiteral(this.ds, node, "code", this.loc.localize(oCode));
    mgStorage.assertLiteral(this.ds, node, "name", oName);
    mgStorage.assertLiteral(this.ds, node, "oRes", mUri);
  },

  getLocNode : function(aNode) {
    var v = aNode.Value;
    if(v && v.substring(0,4) != "urn:") {
      return mgStorage.RDF.GetResource(this.prefix + v);
    } else {
      return aNode;
    }
  },

  isContainerMembership : function(aProperty) {
    // property prefix indicating container membership (followed by number)
    const p = "http://www.w3.org/1999/02/22-rdf-syntax-ns#_";
    return (aProperty.Value.substring(0, p.length) == p);
  },


  // nsIRDFDatasource

  GetSource : function(aProperty, aTarget, aTruthValue) {
    return this.ds.GetSource(aProperty, aTarget, aTruthValue);
  },
  GetSources : function(aProperty, aTarget, aTruthValue) {
    return this.ds.GetSources(aProperty, aTarget, aTruthValue);
  },
  GetTarget : function(aSource, aProperty, aTruthValue) {
    return this.ds.GetTarget(aSource, aProperty, aTruthValue);
  },
  GetTargets: function(aSource, aProperty, aTruthValue) {
    return this.ds.GetTargets(aSource, aProperty, aTruthValue);
  },
  HasAssertion: function(aSource, aProperty, aTarget, aTruthValue) {
    return this.ds.HasAssertion(aSource, aProperty, aTarget, aTruthValue);
  },
  AddObserver: function(aObserver) {
    return this.ds.AddObserver(aObserver);
  },
  RemoveObserver: function(aObserver) {
    return this.ds.RemoveObserver(aObserver);
  },
  ArcLabelsIn: function(aNode) {
    return this.ds.ArcLabelsIn(aNode);
  },
  hasArcOut: function(aSource, aArc) {
    return this.ds.hasArcOut(aSource, aArc);
  },

  Assert: function(aSource, aProperty, aTarget, aTruthValue) {dump("d");},
  Unassert: function(aSource, aProperty, aTarget) {dump("e");},
  Change: function(aSource, aProperty, aOldTarget, aNewTarget) {dump("f");},
  Move: function(aOldSource, aNewSource, aProperty, aTarget) {dump("g");},

  ArcLabelsOut: function(aSource) {
    return this.ds.ArcLabelsOut(aSource);
  },
  GetAllResources : function() {
    return this.ds.GetAllResources();
  },
  IsCommandEnabled: function(aSources, aCommand, aArguments) {
    return this.ds.IsCommandEnabled(aSources, aCommand, aArguments);
  },
  DoCommand: function(aSources, aCommand, aArguments) {
    return this.ds.DoCommand(aSources, aCommand, aArguments);
  },
  GetAllCmds: function(aSource) {
    return this.ds.GetAllCmds(aSource);
  },
  hasArcIn: function(aNode, aArc) {
    return this.ds.hasArcIn(aNode, aArc);
  },

  beginUpdateBatch: function() {
  },
  endUpdateBatch: function() {
  },

  
  // nsIRDFObserver

  onAssert: function(aDataSource, aSource, aProperty, aTarget) {
    if(aSource && aTarget && aTarget.Value) {
      aSource = this.getLocNode(aSource);
      if(mgStorage.RDFCUtils.IsSeq(aDataSource, aSource)
         && this.isContainerMembership(aProperty)) {
        // okay, aTarget is a mapping. copy it
        var container = mgStorage.newContainer();
        container.Init(this.ds, aSource);
        this.copyMapping(aTarget, container);
      }
    }
  },

  onUnassert : function(aDataSource, aSource, aProperty, aTarget) {
    if(aSource && aTarget) {
      aSource = this.getLocNode(aSource);
      if(mgStorage.RDFCUtils.IsSeq(aDataSource, aSource)
         && this.isContainerMembership(aProperty)) {
        // okay, aTarget is a non-literal node (a mapping). translate its URI
        aTarget = this.getLocNode(aTarget);

        // remove it from the container (same URI as in userstore)
        var container = mgStorage.newContainer();
        container.Init(this.ds, aSource);
        container.RemoveElement(aTarget, true);

        // and remove all its properties
        var arcs = this.ds.ArcLabelsOut(aTarget);
        while(arcs.hasMoreElements()) {
          var arc = arcs.getNext();
          var targets = this.ds.GetTargets(aTarget, arc, true);
          while (targets.hasMoreElements()) {
            var target = targets.getNext();
            this.ds.Unassert(aTarget, arc, target, true);
          }
        }
      }
    }
  },

  onChange: function(aDataSource, aSource, aProperty, aTarget, aOldTarget,
                     aNewTarget) {
  },

  onMove: function(aDataSource, aOldSource, aNewSource, aProperty, aTarget) {
  },

}

